from tornado import web, ioloop, routing
from .router import router as mvc_router
from .config import configuration
import sys

class main(web.RequestHandler):
    def prepare(self):
        self.isExec = True
        for middlewareName in configuration.middleware_list:
            middleware = getattr(__import__(middlewareName), middlewareName)(self)
            if hasattr(middleware, "prepare"):
                self.isExec = getattr(middleware, "prepare")()
                if not self.isExec:
                    break

    def on_finish(self):
        if self.isExec:
            for middlewareName in configuration.middleware_list:
                middleware = getattr(__import__(middlewareName), middlewareName)(self)
                if hasattr(middleware, "finish"):
                    getattr(middleware, "finish")()

    def get(self):
        if self.isExec:
            self.router()

    def post(self):
        if self.isExec:
            self.router()

    def router(self):
        try:
            router = mvc_router(self.request.uri)
            controller = getattr(__import__(router.controller), router.controller)(self)
            getattr(controller, router.action)()
        except Exception as e:
            self.set_status(500, e)

class application():
    def __init__(self, appPath, config, port=None):
        self.port = port
        configuration.init(config, appPath)
        self._init_router()
        self._init_path()
        self._run_plugins()

    def _init_router(self):
        self.router = [(r".*", main)]

    def _init_path(self):
        sys.path.insert(0, configuration.controller_path)
        sys.path.insert(0, configuration.middleware_path)
        sys.path.insert(0, configuration.plugin_path)
        sys.path.insert(0, configuration.filter_path)

    def _run_plugins(self):
        for plugin in configuration.plugin_list:
            plugin = getattr(__import__(plugin), plugin)
            self._run_plugin(plugin)

    def _run_plugin(self, plugin):
        plugin(self)

    def run(self):
        self.app = web.Application(self.router)
        self.app.listen(configuration.port if not self.port else self.port)
        ioloop.IOLoop.current().start()


class _configuration(type):
    @staticmethod
    def __getattr__(name):
        value = None
        if hasattr(configuration.config, name):
            value = getattr(configuration.config, name)
        elif hasattr(configuration.default, name):
            value = getattr(configuration.default, name)
        return value.format(appPath=configuration.appPath) if isinstance(value, str) else value

class configuration(metaclass=_configuration):
    __metaclass__ = _configuration

    @staticmethod
    def init(config, appPath):
        configuration.config = config
        configuration.default = deflautConfig()
        configuration.appPath = appPath

class deflautConfig:

    middleware_list = []

    middleware_path = "{appPath}/middleware"

    controller_path = "{appPath}/controller"

    plugin_path = "{appPath}/plugin"

    plugin_list = []

    filter_path = "{appPath}/filter"

    filter_list = []

    cors = "http://localhost"

    cookie_id = "alauda_app_id"

    encrypt = "keyskeyskeyskeys"

    router_controller = "home"

    router_action = "index"


import logging
from .config import configuration

logging.basicConfig(level=configuration.log_level,
                format='%(asctime)s %(filename)s[line:%(lineno)d] %(levelname)s %(message)s',
                datefmt='%a, %d %b %Y %H:%M:%S',
                filename=configuration.log_file,
                filemode='w')

logger = logging

def ErrorLog(func):
    def _dec(*args, **kwargs):
        try:
            func(*args, **kwargs)
        except Exception as e:
            logger.error("An error has occurr in the {func}, error:{e}".format(**locals()))
            raise e
    return _dec


from .config import configuration
import threading

localThread = threading.local()

def filter(name, argv=None):
    filters = {}
    for f in configuration.filter_list:
        filters[f] = getattr(__import__(f, f), f)
    def _filter(func):
        def __filter(*args, **kwargs):
            if filters[name](argv):
                return func(*args, **kwargs)
            else:
                ContextManager.getRequest().set_status(401, "你没有访问权限")
        return __filter
    return _filter

class ContextManager:
    @staticmethod
    def setRequest(request):
        localThread.request = request

    @staticmethod
    def getRequest():
        return localThread.request


#coding: utf8
import sys
from Crypto.Cipher import AES
from binascii import b2a_hex, a2b_hex
from tornado import escape
from .config import configuration
import os, io

class controller():
    _401 = "<!DOCTYPE html><html lang='en'><meta name='viewport' content='width=device-width, initial-scale=1.0'><meta http-equiv='X-UA-Compatible' content='ie=edge'><head><title>401</title><style>html,body,body>div{width:100%;height:100%;margin:0;padding:0;overflow:hidden;text-align:center;color:#527bcc;font:'华文彩云'}h1{margin-top:100px;font-size:160px}.onlinefs-message-one{display:block;font:40px '华文彩云'}.onlinefs-message-two{margin-top:24px;margin-bottom:24px;display:block;font:18px '华文彩云'}a{display:inline-block;height:40px;width:130px;background:#527bcc;font-size:22px;font-family:'华文彩云';text-decoration:none;color:#fff;line-height:40px}</style></head><body><div><h1>401</h1><div class='onlinefs-message-one'>你没有权限访问这个站点，请先申请权限后再登录！</div><div class='onlinefs-message-two'>快回首页试试吧！</div><a href='/'>去首页</a></div></body></html>"

    def __init__(self, application):
        self.application = application
        self.application.set_header("Access-Control-Allow-Origin", configuration.cors)
        self.application.set_header("Access-Control-Allow-Credentials", "true")

    def json_encode(self, value):
        return escape.json_encode(value)

    def json_decode(self, value):
        return escape.json_decode(value)

    def json(self, value):
        json = self.json_encode(value)
        self.application.set_header("Content-Type", "application/json")
        self.application.write(json)

    def success(self, data = None):
        self.json({
            "isSuccess": True,
            "data": data
        })

    def fail(self, message):
        self.json({
            "isSuccess": False,
            "message": message
        })

    def result(self, result, data = None, message = None):
        if result:
            self.success(data)
        else:
            self.fail(message)

    def file(self, path, filename):
        self.application.set_header('Content-Type', 'application/octet-stream')
        self.application.set_header('Content-Disposition', 'attachment; filename=%s' % filename)
        with open(path, 'rb') as f:
            while True:
                data = f.read(4096)
                if not data:
                    break
                self.application.write(data)

    def stream(self, filename, data):
        self.application.set_header('Content-Type', 'application/octet-stream')
        self.application.set_header('Content-Disposition', 'attachment; filename=%s' % filename)
        self.application.write(data)

    def csv(self, filename, fields, rows):
        content = ",".join(fields) + "\n"
        for row in rows:
            tempRow = []
            for field in fields:
                tempRow.append(str(row[field]))
            content += ",".join(tempRow) + "\n"
        self.stream(filename, content.encode("utf-8"))

    def set_cookie(self, name, value):
        self.application.set_cookie(name, value)

    def get_cookie(self, name):
        return self.application.get_cookie(name)

    def __getattr__(self, name):
        value = self.application.get_argument(name)
        try:
            return self.json_decode(value)
        except:
            return value
        
    def set_status(self, code, message):
        self.application.set_status(code, message)
        if code >= 400:
            # self.application.set_header("Content-Type", "application/json")
            # self.json({
            #     "status_code": code,
            #     "message": message
            # })
            self.application.set_header("Content-Type", "text/html; charset=utf-8")
            self.application.write(controller._401)

    @property
    def user(self):
        try:
            return self.decrypt(self.get_cookie(configuration.cookie_id))
        except Exception as e:
            return None

    @property
    def files(self):
        return self.application.request.files.get('file', None)

    def redirect(self, url):
        self.application.redirect(url)

    #加密函数，如果text不是16的倍数【加密文本text必须为16的倍数！】，那就补足为16的倍数
    def encrypt(self, text):
        cryptor = AES.new(configuration.encrypt, AES.MODE_CBC, configuration.encrypt)
        #这里密钥key 长度必须为16（AES-128）、24（AES-192）、或32（AES-256）Bytes 长度.目前AES-128足够用
        length = 16
        count = len(text)
        add = length - (count % length)
        text = text + ('\0' * add)
        ciphertext = cryptor.encrypt(text)
        #因为AES加密时候得到的字符串不一定是ascii字符集的，输出到终端或者保存时候可能存在问题
        #所以这里统一把加密后的字符串转化为16进制字符串
        return b2a_hex(ciphertext)

    def decrypt(self, text):
        cryptor = AES.new(configuration.encrypt, AES.MODE_CBC, configuration.encrypt)
        plain_text = cryptor.decrypt(a2b_hex(text))
        return plain_text.decode().rstrip('\0')


import _thread
import time


class plugin:
    def run(self):
        pass

    def start(self, isOnce, interval):
        if not isOnce:
            _thread.start_new_thread(self._run, (interval,))
        else:
            self.run()

    def _run(self, interval):
        while True:
            self.run()
            time.sleep(interval)


from .config import configuration

class router():
    def __init__(self, url):
        urls = url.split("?")[0].split("/")
        self.controller = configuration.router_controller
        self.action = configuration.router_action
        if len(urls) == 2:
            self.controller = urls[1]
        elif len(urls) == 3:
            self.controller = urls[1]
            self.action = urls[2]
